package compbio.ws.execstat;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import compbio.ws.client.Services;
import compbio.ws.execstat.StatProcessor.JobStat;

public class StatDB {

	/* the default framework is embedded */
	// private final String framework = "embedded";
	private static final String driver = "org.apache.derby.jdbc.EmbeddedDriver";
	private static final String protocol = "jdbc:derby:";
	private static final String statDBName = "ExecutionStatistic";

	private static Connection getDBConnection() throws SQLException {
		// TODO
		System.setProperty("derby.system.home", ".");
		Connection conn = DriverManager.getConnection(protocol + statDBName
				+ ";create=true");

		// We want to control transactions manually. Autocommit is on by
		// default in JDBC.
		// conn.setAutoCommit(false);
		return conn;
	}

	// ServiceName,jobname,start,finish,inputSize,resultSize,isCancelled,isCollected
	/**
	 * 
	 * rs.getBoolean(i) will return true for any non-zero value and false for 0
	 * on SMALLINT data column.
	 * 
	 * @throws SQLException
	 */
	private static void createStatTable() throws SQLException {
		Connection conn = getDBConnection();
		/*
		 * Creating a statement object that we can use for running various SQL
		 * statements commands against the database.
		 */
		Statement s = conn.createStatement();
		String create = "create table exec_stat("
				+ "number INT GENERATED ALWAYS AS IDENTITY,"
				+ "service_name VARCHAR(15) NOT NULL, "
				+ "cluster_job_id VARCHAR(30), "
				+ "job_id VARCHAR(35) NOT NULL PRIMARY KEY, "
				+ "start TIMESTAMP," + "finish TIMESTAMP,"
				+ "inputsize BIGINT," + "resultsize BIGINT,"
				+ "isCancelled SMALLINT NOT NULL,"
				+ "isCollected SMALLINT NOT NULL, "
				+ "isClusterJob SMALLINT NOT NULL)";
		// We create a table...
		System.out.println(create);
		s.execute(create);
		s.close();
		conn.close();
	}

	static void insertData(Set<JobStat> jobstatus) throws SQLException {
		System.out.println("Inserting " + jobstatus.size());
		Connection conn = getDBConnection();
		conn.setAutoCommit(false);
		String insert = "insert into exec_stat (service_name, cluster_job_id, job_id, start, finish, "
				+ "inputsize, resultsize, isCancelled, isCollected, isClusterJob) "
				+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		PreparedStatement pstm = conn.prepareStatement(insert);
		for (JobStat js : jobstatus) {
			pstm.setString(1, js.webService.toString());
			pstm.setString(2, js.clusterJobId);
			pstm.setString(3, js.jobname);
			pstm.setTimestamp(4, new Timestamp(js.start));
			pstm.setTimestamp(5, new Timestamp(js.finish));
			pstm.setLong(6, js.inputSize);
			pstm.setLong(7, js.resultSize);
			pstm.setBoolean(8, js.isCancelled);
			pstm.setBoolean(9, js.isCollected);
			pstm.setBoolean(10, js.isClusterJob());
			pstm.executeUpdate();
		}
		conn.commit();
		pstm.close();
		conn.close();
	}

	static List<JobStat> readData(Timestamp from, Timestamp to,
			Services wservice, Boolean clusterOnly) throws SQLException {
		Connection conn = getDBConnection();
		String query = "select service_name, cluster_job_id, job_id, start, finish, inputsize, "
				+ "resultsize, isCancelled, isCollected from exec_stat where start BETWEEN ? and ? ";

		if (wservice != null) {
			query += " and service_name=? ";
		}

		if (clusterOnly != null) {
			if (clusterOnly) {
				query += " and isClusterJob!=0 ";
			} else {
				query += " and isClusterJob=0 ";
			}
		}

		PreparedStatement pstm = conn.prepareStatement(query);
		pstm.setTimestamp(1, from);
		pstm.setTimestamp(2, to);
		if (wservice != null) {
			pstm.setString(3, wservice.toString());
		}
		pstm.execute();
		List<JobStat> stats = new ArrayList<StatProcessor.JobStat>();
		ResultSet rs = pstm.getResultSet();
		while (rs.next()) {
			stats.add(new JobStat(Services.getService(rs.getString(1)), rs
					.getString(2), rs.getString(3), rs.getTimestamp(4)
					.getTime(), rs.getTimestamp(5).getTime(), rs.getLong(6), rs
					.getLong(7), rs.getBoolean(8), rs.getBoolean(9)));
		}
		rs.close();
		pstm.close();
		conn.close();
		return stats;
	}

	static void removeRecordedJobs(Set<String> fsJobs) throws SQLException {
		Connection conn = getDBConnection();
		String query = "select job_id from exec_stat";

		Statement st = conn.createStatement();
		ResultSet result = st.executeQuery(query);

		while (result.next()) {
			String recordedJob = result.getString(1);
			if (fsJobs.contains(recordedJob)) {
				fsJobs.remove(recordedJob);
			}
		}
		result.close();
		conn.close();
	}

	void shutdownDBServer() {
		// ## DATABASE SHUTDOWN SECTION ##
		/***
		 * In embedded mode, an application should shut down Derby. Shutdown
		 * throws the XJ015 exception to confirm success.
		 ***/
		boolean gotSQLExc = false;
		try {
			DriverManager.getConnection("jdbc:derby:;shutdown=true");
		} catch (SQLException se) {
			if (se.getSQLState().equals("XJ015")) {
				gotSQLExc = true;
			}
		}
		if (!gotSQLExc) {
			System.out.println("Database did not shut down normally");
		} else {
			System.out.println("Database shut down normally");
		}
	}
	public static void main(String[] args) throws SQLException {
		// createStatTable();
		// insertData(null);

		Date from = new Date();
		from.setMonth(1);
		System.out.println(new StatProcessor(readData(
				new Timestamp(from.getTime()),
				new Timestamp(new Date().getTime()), null, null)).reportStat());

	}
}
